/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "entity.h"
#include "game/game.h"
#include "util/memory.h"
#include "assert/assert.h"
#include "display/render.h"
#include "input.h"

void entityInit(entity_t *entity, const entitytype_t type) {
  memoryZero(entity, sizeof(entity_t));

  entity->type = type;
  switch(type) {
    case ENTITY_TYPE_PLAYER:
      break;
    
    default:
      assertUnreachable("Unknown entity type.");
      break;
  }
}

void entityTest(entity_t *ent) {
  if(inputIsPressed(INPUT_DOWN)) {
    ent->direction = DIRECTION_DOWN;
    if(ent->y < GAME.map.height - 1) ent->y++;
  } else if(inputIsPressed(INPUT_UP)) {
    ent->direction = DIRECTION_UP;
    if(ent->y > 0) ent->y--;
  } else if(inputIsPressed(INPUT_LEFT)) {
    ent->direction = DIRECTION_LEFT;
    if(ent->x > 0) ent->x--;
  } else if(inputIsPressed(INPUT_RIGHT)) {
    ent->direction = DIRECTION_RIGHT;
    if(ent->x < GAME.map.width - 1) ent->x++;
  }
}

void entityRender(const entity_t *ent) {
  // Draw the entity
  int32_t fontSize = 20;
  char_t str[1];
  switch(ent->direction) {
    case DIRECTION_NORTH:
      str[0] = '^';
      break;
    case DIRECTION_EAST:
      str[0] = '>';
      break;
    case DIRECTION_SOUTH:
      str[0] = 'v';
      break;
    case DIRECTION_WEST:
      str[0] = '<';
      break;
    default:
      assertUnreachable("Invalid entity direction.");
      break;
  }

  DrawTextEx(
    GetFontDefault(),
    str,
    (struct Vector2){
      ent->x * RENDER_FONT_SIZE,
      ent->y * RENDER_FONT_SIZE
    },
    RENDER_FONT_SIZE,
    (float)(RENDER_FONT_SIZE / 10),
    WHITE
  );
}