/**
 * Copyright (c) 2025 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "packetqueue.h"
#include "assert/assert.h"
#include "util/memory.h"

void packetQueueInit(packetqueue_t *queue) {
  assertNotNull(queue, "Packet queue is NULL");
  memoryZero(queue, sizeof(packetqueue_t));
  pthread_mutex_init(&queue->lock, NULL);
}

void packetQueuePushIn(packetqueue_t *queue, const packet_t *packet) {
  assertNotNull(queue, "Packet queue is NULL");
  assertNotNull(packet, "Packet is NULL");
  pthread_mutex_lock(&queue->lock);
  assertTrue(
    queue->packetsInCount < PACKET_QUEUE_MAX_SIZE, "Inbound packet queue is full"
  );

  queue->packetsIn[queue->packetsInCount++] = *packet;
  pthread_mutex_unlock(&queue->lock);
}

void packetQueuePushOut(packetqueue_t *queue, const packet_t *packet) {
  assertNotNull(queue, "Packet queue is NULL");
  assertNotNull(packet, "Packet is NULL");
  pthread_mutex_lock(&queue->lock);
  assertTrue(
    queue->packetsOutCount < PACKET_QUEUE_MAX_SIZE, "Outbound packet queue is full"
  );

  queue->packetsOut[queue->packetsOutCount++] = *packet;
  pthread_mutex_unlock(&queue->lock);
}

int32_t packetQueuePopIn(packetqueue_t *queue, packet_t *packet) {
  assertNotNull(queue, "Packet queue is NULL");
  assertNotNull(packet, "Packet is NULL");

  pthread_mutex_lock(&queue->lock);
  if(queue->packetsInCount == 0) {
    pthread_mutex_unlock(&queue->lock);
    return 0;
  }

  *packet = queue->packetsIn[0];
  if(queue->packetsInCount > 1) {
    memoryCopy(
      &queue->packetsIn[0],
      &queue->packetsIn[1],
      (queue->packetsInCount - 1) * sizeof(packet_t)
    );
  }
  queue->packetsInCount--;
  pthread_mutex_unlock(&queue->lock);
  return 1;
}

int32_t packetQueuePopOut(packetqueue_t *queue, packet_t *packet) {
  assertNotNull(queue, "Packet queue is NULL");
  assertNotNull(packet, "Packet is NULL");

  pthread_mutex_lock(&queue->lock);
  if(queue->packetsOutCount == 0) {
    pthread_mutex_unlock(&queue->lock);
    return 0;
  }

  *packet = queue->packetsOut[0];
  if(queue->packetsOutCount > 1) {
    memoryCopy(
      &queue->packetsOut[0],
      &queue->packetsOut[1],
      (queue->packetsOutCount - 1) * sizeof(packet_t)
    );
  }
  queue->packetsOutCount--;
  pthread_mutex_unlock(&queue->lock);
  return 1;
}